@@ -1,8 +1,8 @@
 ;;;============================================================================
 
-;;; File: "_ptree1.scm", Time-stamp: <2008-09-12 16:54:15 feeley>
+;;; File: "_ptree1.scm", Time-stamp: <2009-06-07 20:28:06 feeley>
 
-;;; Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2009 by Marc Feeley, All Rights Reserved.
 
 (include "fixnum.scm")
 
@@ -473,6 +473,9 @@
 (define (inline-primitive? name env) ; true iff name can be inlined
   (declaration-value inline-primitives-sym name #t env))
 
+(define (add-not-inline-primitive? env)
+  (env-declare env (list inline-primitives-sym #f)))
+
 (define (inlining-limit env) ; returns the inlining limit
   (max 0 (min 1000000 (declaration-value inlining-limit-sym #f 350 env))))
 
@@ -562,7 +565,7 @@
                  lst
                  proc))
 
-              ((**begin-expr? source)
+              ((**begin-cmd-or-expr? source)
                (parse-prog
                  (append (begin-body source) (cdr program))
                  env
@@ -1347,7 +1350,7 @@
                            (cons (macro-expand (car body) env)
                                  (cdr body))
                            env))
-          ((**begin-expr? (car body))
+          ((**begin-cmd-or-expr? (car body))
            (letrec-defines vars
                            vals
                            envs
@@ -1413,11 +1416,15 @@
   (letrec-defines '() '() '() body (env-frame env '())))
 
 (define (pt-sequence source seq env use)
-  (if (length? seq 1)
-    (pt (car seq) env use)
-    (new-seq source env
-      (pt (car seq) env 'none)
-      (pt-sequence source (cdr seq) env use))))
+  (cond ;; ((length? seq 0)
+        ;; ;; treat empty sequence as constant evaluating to the void object
+        ;; (new-cst source env void-object))
+        ((length? seq 1)
+         (pt (car seq) env use))
+        (else
+         (new-seq source env
+           (pt (car seq) env 'none)
+           (pt-sequence source (cdr seq) env use)))))
 
 (define (pt-if source env use)
   (let ((code (source-code source)))
@@ -1912,9 +1919,12 @@
                           (ill-formed-special-form source)))
                     (ill-formed-special-form source))))))))
 
-(define (**begin-expr? source)
+(define (**begin-cmd-or-expr? source)
   (match **begin-sym -1 source))
 
+(define (**begin-expr? source)
+  (match **begin-sym -2 source))
+
 (define (**define-expr? source env)
   (match **define-sym -2 source))
 
