@@ -1,14 +1,11 @@
 ;;;============================================================================
 
-;;; File: "main.scm", Time-stamp: <2008-10-29 10:33:49 feeley>
+;;; File: "main.scm", Time-stamp: <2009-06-07 19:14:45 feeley>
 
-;;; Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2009 by Marc Feeley, All Rights Reserved.
 
 ;;;----------------------------------------------------------------------------
 
-(##define-macro (macro-extension-file)
-  "gambcext")
-
 (##define-macro (macro-initialization-file)
   ".gambcini")
 
@@ -199,8 +196,8 @@
               (if (or pre post)
                 (set! c#wrap-program
                   (lambda (program)
-                    (let ((file
-                           (##container->file
+                    (let ((path
+                           (##container->path
                             (##locat-container
                              (##source-locat program)))))
                       (##sourcify
@@ -210,9 +207,9 @@
                                   (let ((pre-src
                                          (read-source-from-string
                                           (##cdr pre)
-                                          (and file
+                                          (and path
                                                (##string-append
-                                                file
+                                                path
                                                 ".prelude")))))
                                     (##list pre-src))
                                   '())
@@ -221,9 +218,9 @@
                                           (let ((post-src
                                                  (read-source-from-string
                                                   (##cdr post)
-                                                  (and file
+                                                  (and path
                                                        (##string-append
-                                                        file
+                                                        path
                                                         ".postlude")))))
                                             (##list post-src))
                                           '()))))
@@ -293,7 +290,7 @@
                                         options: sym-opts))))
                                 (##exit-abnormally))
                             (loop2 rest
-                                   (##cons (##path-strip-directory root)
+                                   (##cons root
                                            rev-roots)))))))
 
                   (let* ((flat?
@@ -423,12 +420,7 @@
 
         (cont (##reverse rev-options) args))))
 
-  (let ((gambcdir (##path-expand "~~")))
-    (##load (##string-append gambcdir (macro-extension-file))
-            (lambda (script-line script-path) #f)
-            #f
-            #f
-            #f))
+  (##load-support-libraries)
 
   (let ((language-and-tail
          (##extract-language-and-tail (##car ##processed-command-line))))
@@ -453,7 +445,11 @@
             (begin
               (##write-string (##system-version-string) ##stdout-port)
               (##write-string " " ##stdout-port)
-              (##write (##list (##system-stamp)) ##stdout-port)
+              (##write (##system-stamp) ##stdout-port)
+              (##write-string " " ##stdout-port)
+              (##write-string ##os-system-type-string-saved ##stdout-port)
+              (##write-string " " ##stdout-port)
+              (##write ##os-configure-command-string-saved ##stdout-port)
               (##newline ##stdout-port)
               (##exit))
             (split-command-line
