@@ -1,8 +1,8 @@
 ;;;============================================================================
 
-;;; File: "_io#.scm", Time-stamp: <2008-11-23 00:45:36 feeley>
+;;; File: "_io#.scm", Time-stamp: <2009-02-20 13:57:20 feeley>
 
-;;; Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2009 by Marc Feeley, All Rights Reserved.
 
 ;;;============================================================================
 
@@ -486,7 +486,7 @@
 
   (name      printable: read-only:)
   (aliases   printable: read-only:)
-  (net       printable: read-only:)
+  (number    printable: read-only:)
 )
 
 ;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -707,15 +707,15 @@
 (##define-macro (macro-char-encoding-ISO-8859-1) 2)
 (##define-macro (macro-char-encoding-UTF-8)      3)
 (##define-macro (macro-char-encoding-UTF-16)     4)
-(##define-macro (macro-char-encoding-UTF-16LE)   5)
-(##define-macro (macro-char-encoding-UTF-16BE)   6)
+(##define-macro (macro-char-encoding-UTF-16BE)   5)
+(##define-macro (macro-char-encoding-UTF-16LE)   6)
 (##define-macro (macro-char-encoding-UTF)        7)
 (##define-macro (macro-char-encoding-UCS-2)      8)
-(##define-macro (macro-char-encoding-UCS-2LE)    9)
-(##define-macro (macro-char-encoding-UCS-2BE)    10)
+(##define-macro (macro-char-encoding-UCS-2BE)    9)
+(##define-macro (macro-char-encoding-UCS-2LE)    10)
 (##define-macro (macro-char-encoding-UCS-4)      11)
-(##define-macro (macro-char-encoding-UCS-4LE)    12)
-(##define-macro (macro-char-encoding-UCS-4BE)    13)
+(##define-macro (macro-char-encoding-UCS-4BE)    12)
+(##define-macro (macro-char-encoding-UCS-4LE)    13)
 (##define-macro (macro-char-encoding-wchar)      14)
 (##define-macro (macro-char-encoding-native)     15)
 
@@ -891,6 +891,7 @@
   marktable
   force?
   width
+  shift
   close-parens
   level
   limit
@@ -922,6 +923,7 @@
   unwrapper
   allow-script?
   labels
+  container
   filepos
 )
 
