@@ -1,6 +1,6 @@
-/* File: "os.h", Time-stamp: <2008-09-03 16:32:13 feeley> */
+/* File: "os.h", Time-stamp: <2009-06-04 19:30:18 feeley> */
 
-/* Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2009 by Marc Feeley, All Rights Reserved. */
 
 #ifndef ___OS_H
 #define ___OS_H
@@ -57,32 +57,42 @@
 #endif
 #endif
 
-#ifdef USE_GENERIC_OS
+/*---------------------------------------------------------------------------*/
 
-/* Basic features we require */
+/*
+ * We assume that the following basic features are available
+ * regardless of the operating-system... otherwise we are in real
+ * trouble!
+ */
 
 #define USE_errno
 #define USE_malloc
 #define USE_memmove
 
+
+/*
+ * The following basic features are used if they are available.
+ */
+
+#ifdef HAVE_GETENV
+#define USE_getenv
 #endif
 
+#ifdef HAVE_SETENV
+#define USE_setenv
+#endif
+
+#ifdef HAVE_UNSETENV
+#define USE_unsetenv
+#endif
+
+
+/* Operating-system specific features we require */
+
 #ifdef USE_POSIX
 
-/* Basic features we require */
-
-#define USE_errno
-#define USE_malloc
-#define USE_memmove
-#define USE_open
-
-#if 1
 #define USE_environ
-#else
-#define USE_getenv
-#define USE_setenv
-#define USE_unsetenv
-#endif
+#define USE_open
 
 /* Select features based on availability */
 
@@ -151,8 +161,18 @@
 #define USE_NETWORKING
 #endif
 
+#ifdef HAVE_STAT64
+#define USE_stat
+#define ___stat stat64
+#define ___lstat lstat64
+#define ___fstat fstat64
+#else
 #ifdef HAVE_STAT
 #define USE_stat
+#define ___stat stat
+#define ___lstat lstat
+#define ___fstat fstat
+#endif
 #endif
 
 #ifdef HAVE_STRERROR
@@ -206,10 +226,6 @@
 
 #undef LINEEDITOR_WITH_LOCAL_CLIPBOARD
 
-#define USE_errno
-#define USE_malloc
-#define USE_memmove
-
 #define USE_ioctl
 #define USE_CopyFile
 #define USE_CreateDirectory
@@ -228,6 +244,7 @@
 #define USE_SetEnvironmentVariable
 #define USE_WSAGetLastError
 #define USE_GetConsoleWindow
+#define USE_GetModuleFileName
 
 #define HAVE_CLOCK 1
 #define HAVE_CREATETHREAD 1
@@ -316,6 +333,7 @@
 #endif
 #endif
 
+
 /* Determine which function for getting process time is most precise.  */
 
 #ifdef HAVE_GETPROCESSTIMES
@@ -968,6 +986,9 @@ ___END_C_LINKAGE
 
 #ifdef INCLUDE_sys_stat_h
 #ifdef HAVE_SYS_STAT_H
+/* the following defines are useful on Linux to map stat to stat64 */
+#define __USE_LARGEFILE64
+#define __USE_FILE_OFFSET64
 #include <sys/stat.h>
 #endif
 #endif
@@ -1340,6 +1361,7 @@ extern char *___format_filepos
 
 extern char **___os_system_type ___PVOID;
 extern char *___os_system_type_string ___PVOID;
+extern char *___os_configure_command_string ___PVOID;
 
 
 /* OS initialization/finalization. */
