@@ -1,13 +1,13 @@
-/* File: "os_base.c", Time-stamp: <2008-05-17 08:22:56 feeley> */
+/* File: "os_base.c", Time-stamp: <2009-03-18 09:25:14 feeley> */
 
-/* Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2009 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the most basic operating system services.
  */
 
 #define ___INCLUDED_FROM_OS_BASE
-#define ___VERSION 403001
+#define ___VERSION 404004
 #include "gambit.h"
 
 #include "os_base.h"
@@ -117,10 +117,8 @@ ___FILE *stream;)
 int ___printf
    ___P((const char *format,
          ...),
-        (format,
-         va_alist)
-const char *format;
-va_dcl)
+        (format, ...)
+const char *format;)
 {
   va_list ap;
   int result;
@@ -186,19 +184,6 @@ unsigned long bytes;)
 
   ptr = malloc (bytes);
 
-#endif
-
-#ifdef ___DEBUG
-#ifdef ___DEBUG_ALLOC_MEM_TRACE
-  if (___base_mod.file != 0)
-    ___printf ("%p (%lu bytes) ALLOCATED AT \"%s\"@%d.1\n",
-               ptr,
-               bytes,
-               ___base_mod.file,
-               ___base_mod.lineno);
-  else
-    ___printf ("%p (%lu bytes) ALLOCATED\n", ptr, bytes);
-#endif
 #endif
 
   return ptr;
@@ -265,13 +250,16 @@ char *file;)
 {
   void *ptr;
 
-  ___base_mod.lineno = lineno;
-  ___base_mod.file = file;
-
   ptr = ___alloc_mem (bytes);
 
-  ___base_mod.lineno = 0;
-  ___base_mod.file = 0;
+  if (file != 0)
+    ___printf ("%p (%lu bytes) ALLOCATED AT \"%s\"@%d.1\n",
+               ptr,
+               bytes,
+               file,
+               lineno);
+  else
+    ___printf ("%p (%lu bytes) ALLOCATED\n", ptr, bytes);
 
   return ptr;
 }
@@ -286,10 +274,20 @@ char *file;)
 /* Program startup */
 
 
+/*
+ * ___main_char, ___main_UCS_2, and ___winmain are variants of main
+ * entry points which differ in the format of the arguments; the first
+ * two are always compiled in since they could be useful on all
+ * systems for embedding of Gambit.
+ */
+
+
+/* To keep command line and runtime flag information around: */
+
 ___program_startup_info_struct ___program_startup_info =
 {
-  0,
-  0
+  0, /* argument vector */
+  0  /* runtime flag string */
 
 #ifdef ___OS_WIN32
   ,
@@ -1439,7 +1437,7 @@ ___SCMOBJ ___setup_base_module ___PVOID
 #endif
 
 #ifdef USE_WIN32
-      ___base_mod.debug = ___fopen ("console", "w");
+      ___base_mod.debug = ___fopen ("con:", "w");
 #endif
 
       if (___base_mod.debug == NULL)
@@ -1450,13 +1448,6 @@ ___SCMOBJ ___setup_base_module ___PVOID
       ___base_mod.alloc_mem_calls = 0;
       ___base_mod.free_mem_calls = 0;
 
-#ifdef ___DEBUG_ALLOC_MEM_TRACE
-
-      ___base_mod.lineno = 0;
-      ___base_mod.file = 0;
-
-#endif
-
 #endif
 
       setup_fp ();
