@@ -1,6 +1,6 @@
-/* File: "os_files.c", Time-stamp: <2008-05-17 08:19:31 feeley> */
+/* File: "os_files.c", Time-stamp: <2009-03-14 11:14:20 feeley> */
 
-/* Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2009 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the operating system specific routines
@@ -8,7 +8,7 @@
  */
 
 #define ___INCLUDED_FROM_OS_FILES
-#define ___VERSION 403001
+#define ___VERSION 404004
 #include "gambit.h"
 
 #include "os_base.h"
@@ -506,6 +506,56 @@ ___SCMOBJ ___os_path_gambcdir ___PVOID
   ___SCMOBJ e;
   ___SCMOBJ result;
 
+#ifdef USE_WIN32
+#ifndef ___GAMBCDIR
+#ifdef USE_GetModuleFileName
+  if (___setup_params.gambcdir == 0)
+    {
+      ___CHAR_TYPE(___PATH_CE_SELECT) temp[___PATH_MAX_LENGTH+1];
+      DWORD n;
+
+      n = GetModuleFileName (NULL, temp, ___PATH_MAX_LENGTH+1);
+      if (n > 0)
+        {
+	  int cch;
+	  ___UCS_2STRING gambcdir = 0;
+	  /* remove filename */
+	  *(_tcsrchr (temp, '\\')) = 0;
+	  /* remove bin subdirectory, if present */
+	  cch = _tcslen (temp);
+	  if (cch > 7) /* e.g. C:\x\bin */
+            {
+	      if (0 == _tcsicmp (temp+cch-4, _T("\\bin")))
+                {
+		  cch -= 4;
+		  *(temp+cch) = '\0';
+                }
+            }
+
+	  gambcdir = ___CAST(___UCS_2STRING,
+			     ___alloc_mem ((cch+1) * sizeof (___UCS_2)));
+
+	  if (gambcdir == 0)
+            {
+	      e = ___FIX(___HEAP_OVERFLOW_ERR);
+	      return e;
+            }
+	  else
+            {
+#ifdef _UNICODE
+	      _tcscpy (gambcdir, temp);
+#else
+	      mbstowcs (gambcdir, temp, cch);
+              gambcdir[cch] = '\0';
+#endif
+	      ___setup_params.gambcdir = gambcdir;
+            }
+      }
+  }
+#endif
+#endif
+#endif
+
   if (___setup_params.gambcdir != 0)
     {
       if ((e = ___NONNULLUCS_2STRING_to_SCMOBJ
@@ -530,6 +580,7 @@ ___SCMOBJ ___os_path_gambcdir ___PVOID
 #endif
 
 #ifdef USE_WIN32
+/* Will only be used if GetModuleFileName path fails */
 #define ___GAMBCDIR "c:\\Gambit-C\\" STRINGIFY2(___VERSION)
 #endif
 
@@ -553,6 +604,176 @@ ___SCMOBJ ___os_path_gambcdir ___PVOID
 }
 
 
+#ifndef ___GAMBCDIR_MAP_CE_SELECT
+#define ___GAMBCDIR_MAP_CE_SELECT(latin1,utf8,ucs2,ucs4,wchar,native) ucs2
+#endif
+
+#ifndef ___CONFIG_GAMBCDIR_MAP_CE_SELECT
+#define ___CONFIG_GAMBCDIR_MAP_CE_SELECT(latin1,utf8,ucs2,ucs4,wchar,native) native
+#endif
+
+
+/* 
+ * TODO: the current implementation of the lookup duplicates the
+ * lookup logic because the configuration map and the map from the
+ * runtime options are not represented with the same string type.  The
+ * proper approach would be to represent OS environment variables
+ * using UTF-8 strings, but this would require substantial changes.
+ */
+
+
+___HIDDEN ___STRING_TYPE(___GAMBCDIR_MAP_CE_SELECT) gambcdir_map_lookup
+   ___P((___STRING_TYPE(___GAMBCDIR_MAP_CE_SELECT) d),
+        (d)
+___STRING_TYPE(___GAMBCDIR_MAP_CE_SELECT) d;)
+{
+  ___STRING_TYPE(___GAMBCDIR_MAP_CE_SELECT) dir;
+  ___STRING_TYPE(___GAMBCDIR_MAP_CE_SELECT) *p = ___setup_params.gambcdir_map;
+
+  if (p == 0)
+    return 0;
+
+  while ((dir = *p++) != 0)
+    {
+      int i = 0;
+      for (;;)
+        {
+          ___UCS_2 c = d[i];
+          if (c == '\0')
+            {
+              if (dir[i] == '=')
+                return dir+i+1;
+              else
+                break;
+            }
+          else if ((dir[i] == '=') || (dir[i] != c))
+            {
+              break;
+            }
+          i++;
+        }
+    }
+
+  return 0;
+}
+
+
+___HIDDEN ___STRING_TYPE(___CONFIG_GAMBCDIR_MAP_CE_SELECT) config_gambcdir_map[] =
+{
+#ifdef ___GAMBCDIR_BIN
+  "bin=" ___GAMBCDIR_BIN,
+#endif
+#ifdef ___GAMBCDIR_DOC
+  "doc=" ___GAMBCDIR_DOC,
+#endif
+#ifdef ___GAMBCDIR_INCLUDE
+  "include=" ___GAMBCDIR_INCLUDE,
+#endif
+#ifdef ___GAMBCDIR_INFO
+  "info=" ___GAMBCDIR_INFO,
+#endif
+#ifdef ___GAMBCDIR_LIB
+  "lib=" ___GAMBCDIR_LIB,
+#endif
+#ifdef ___GAMBCDIR_SHARE
+  "share=" ___GAMBCDIR_SHARE,
+#endif
+  0
+};
+
+
+___HIDDEN ___STRING_TYPE(___CONFIG_GAMBCDIR_MAP_CE_SELECT) config_gambcdir_map_lookup
+   ___P((___STRING_TYPE(___GAMBCDIR_MAP_CE_SELECT) d),
+        (d)
+___STRING_TYPE(___GAMBCDIR_MAP_CE_SELECT) d;)
+{
+  ___STRING_TYPE(___CONFIG_GAMBCDIR_MAP_CE_SELECT) dir;
+  ___STRING_TYPE(___CONFIG_GAMBCDIR_MAP_CE_SELECT) *p = config_gambcdir_map;
+
+  while ((dir = *p++) != 0)
+    {
+      int i = 0;
+      for (;;)
+        {
+          ___UCS_2 c = d[i];
+          if (c == '\0')
+            {
+              if (dir[i] == '=')
+                return dir+i+1;
+              else
+                break;
+            }
+          else if ((dir[i] == '=') || (dir[i] != c))
+            {
+              break;
+            }
+          i++;
+        }
+    }
+
+  return 0;
+}
+
+
+___SCMOBJ ___os_path_gambcdir_map_lookup
+   ___P((___SCMOBJ dir),
+        (dir)
+___SCMOBJ dir;)
+{
+  ___SCMOBJ e;
+  ___SCMOBJ result;
+  void *cdir;
+
+  if ((e = ___SCMOBJ_to_STRING
+             (dir,
+              &cdir,
+              1,
+              ___CE(___GAMBCDIR_MAP_CE_SELECT),
+              0))
+      != ___FIX(___NO_ERR))
+    result = e;
+  else
+    {
+      ___STRING_TYPE(___GAMBCDIR_MAP_CE_SELECT) d =
+        ___CAST(___STRING_TYPE(___GAMBCDIR_MAP_CE_SELECT),cdir);
+
+      ___STRING_TYPE(___GAMBCDIR_MAP_CE_SELECT) dir1;
+      ___STRING_TYPE(___CONFIG_GAMBCDIR_MAP_CE_SELECT) dir2;
+
+      if ((dir1 = gambcdir_map_lookup (d)) != 0)
+        {
+          if ((e = ___STRING_to_SCMOBJ
+                     (dir1,
+                      &result,
+                      ___RETURN_POS,
+                      ___CE(___GAMBCDIR_MAP_CE_SELECT)))
+              != ___FIX(___NO_ERR))
+            result = e;
+          else
+            ___release_scmobj (result);
+        }
+      else if ((dir2 = config_gambcdir_map_lookup (d)) != 0)
+        {
+          if ((e = ___STRING_to_SCMOBJ
+                     (dir2,
+                      &result,
+                      ___RETURN_POS,
+                      ___CE(___CONFIG_GAMBCDIR_MAP_CE_SELECT)))
+              != ___FIX(___NO_ERR))
+            result = e;
+          else
+            ___release_scmobj (result);
+        }
+      else
+        result = ___FAL;
+
+      ___release_string (cdir);
+    }
+
+  return result;
+}
+
+
 ___SCMOBJ ___os_path_normalize_directory
    ___P((___SCMOBJ path),
         (path)
@@ -579,15 +800,45 @@ ___SCMOBJ path;)
 #ifndef USE_POSIX
 #ifndef USE_WIN32
 
-      if ((e = ___NONNULLSTRING_to_SCMOBJ
-                 ("",
-                  &result,
-                  ___RETURN_POS,
-                  ___CHAR_ENCODING_NATIVE))
-          != ___FIX(___NO_ERR))
-        result = e;
+      ___CHAR_TYPE(___PATH_CE_SELECT) normalized_dir[___PATH_MAX_LENGTH+1+1];
+      ___FILE *exist_check;
+
+      dir = normalized_dir;
+
+      if (p == 0)
+        p = ".";
+
+      while (*p != '\0')
+        *dir++ = *p++;
+
+      if (dir == normalized_dir || dir[-1] != '/')
+        *dir++ = '/';
+
+      *dir++ = '\0';
+
+      dir = normalized_dir;
+
+      while (dir[0] == '.' && dir[1] == '/' && dir[2] != '\0')
+        dir += 2;
+
+      exist_check = ___fopen (dir, "r");
+
+      if (exist_check == 0)
+        result = fnf_or_err_code_from_errno ();
       else
-        ___release_scmobj (result);
+        {
+          ___fclose (exist_check);
+
+          if ((e = ___NONNULLSTRING_to_SCMOBJ
+                     (dir,
+                      &result,
+                      ___RETURN_POS,
+                      ___CE(___PATH_CE_SELECT)))
+              != ___FIX(___NO_ERR))
+            result = e;
+          else
+            ___release_scmobj (result);
+        }
 
 #endif
 #endif
