@@ -1,4 +1,4 @@
-/* File: "setup.c", Time-stamp: <2008-11-23 02:32:28 feeley> */
+/* File: "setup.c", Time-stamp: <2009-06-10 07:25:51 feeley> */
 
 /* Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved. */
 
@@ -8,7 +8,7 @@
  */
 
 #define ___INCLUDED_FROM_SETUP
-#define ___VERSION 403001
+#define ___VERSION 404004
 #include "gambit.h"
 
 #include "os_base.h"
@@ -43,7 +43,7 @@ ___HIDDEN ___UCS_2 reset_argv0[] = { 0 };
 ___HIDDEN ___UCS_2STRING reset_argv[] = { reset_argv0, 0 };
 
 ___setup_params_struct ___setup_params =
-{ 0, reset_argv, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+{ 0, reset_argv, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 
 
 /* 
@@ -1494,7 +1494,15 @@ double x;)
 
 #else
 
-  return ((___FETCH_U16(&x,F64_HI16) ^ 0x7ff0) & 0x7fff) >= 0x10;
+  union
+    {
+      ___U16 u16[4];
+      ___F64 f64;
+    } y;
+
+  y.f64 = x;
+
+  return ((y.u16[F64_HI16] ^ 0x7ff0) & 0x7fff) >= 0x10;
 
 #endif
 }
@@ -1511,9 +1519,19 @@ double x;)
 
 #else
 
-  ___UM32 tmp = (___FETCH_U32(&x,F64_HI32) ^ 0x7ff00000) & 0x7fffffff;
+  ___UM32 tmp;
 
-  return tmp < 0x100000 && (tmp | ___FETCH_U32(&x,F64_LO32)) != 0;
+  union
+    {
+      ___U32 u32[2];
+      ___F64 f64;
+    } y;
+
+  y.f64 = x;
+
+  tmp = (y.u32[F64_HI32] ^ 0x7ff00000) & 0x7fffffff;
+
+  return tmp < 0x100000 && (tmp | y.u32[F64_LO32]) != 0;
 
 #endif
 }
@@ -1986,6 +2004,7 @@ ___setup_params_struct *setup_params;)
   setup_params->terminal_settings = 0;
   setup_params->stdio_settings    = 0;
   setup_params->gambcdir          = 0;
+  setup_params->gambcdir_map      = 0;
   setup_params->remote_dbg_addr   = 0;
   setup_params->rpc_server_addr   = 0;
   setup_params->linker            = 0;
